/*
*   Copyright (c) 2017, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
*
*  WSO2 Inc. licenses this file to you under the Apache License,
*  Version 2.0 (the "License"); you may not use this file except
*  in compliance with the License.
*  You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/
package org.wso2.ballerinalang.compiler.tree.statements;

import org.ballerinalang.model.tree.NodeKind;
import org.ballerinalang.model.tree.statements.ForkJoinNode;
import org.wso2.ballerinalang.compiler.tree.BLangNodeVisitor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Implementation of {@link ForkJoinNode}.
 *
 * @since 0.94
 */
public class BLangForkJoin extends BLangStatement implements ForkJoinNode {

    public List<BLangSimpleVariableDef> workers;

    public BLangForkJoin() {
        this.workers = new ArrayList<>();
    }

    @Override
    public List<BLangSimpleVariableDef> getWorkers() {
        return workers;
    }

    @Override
    public void addWorkers(BLangSimpleVariableDef workerDef) {
        workers.add(workerDef);
    }

    @Override
    public NodeKind getKind() {
        return NodeKind.FORK_JOIN;
    }

    @Override
    public void accept(BLangNodeVisitor visitor) {
        visitor.visit(this);
    }

    @Override
    public String toString() {
        return "BLangForkJoin: " + Arrays.toString(workers.toArray());
    }
}
